
# q-expansion of newform 5544.2.a.q, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5544
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [1], [1], [-5], [6], [-1], [-4], [-1], [-10], [1], [0], [-8], [-1], [-8], [3], [-6], [13], [0], [-3], [-8], [-2], [6], [-16], [-10], [2], [7], [10], [14], [-22], [8], [-6], [0], [-13], [-10], [20], [-19], [-8], [0], [4], [12], [-18], [20], [14], [0], [-18], [-22], [6], [16], [-18], [13], [-1], [-3], [17], [-5], [18], [19], [12], [-13], [-9], [0], [-4], [-32], [-14], [8], [8], [28], [-16], [15], [-5], [4], [-26], [-18], [-21], [-24], [-20], [2], [28], [-14], [-35], [-1], [-21], [28], [-5], [-24], [-24], [-32], [24], [29], [-33], [42], [-32], [15], [-5], [24], [-30], [-39], [1], [20], [28], [-37], [32], [42], [-4], [-2], [41], [36], [-24], [-15], [19], [2], [22], [-46], [20], [-38], [-8], [21], [-16], [19], [36], [-8], [42], [-22], [-6], [22], [-5], [-15], [-8], [2], [0], [51], [21], [17], [24], [-41], [-11], [-7], [19], [31], [25], [-11], [-3], [23], [-16], [-9], [-42], [-30], [24], [36], [0], [33], [3], [14], [-52], [26], [-60], [59], [22], [28], [-42], [-17], [-46], [9], [8], [32], [21], [46], [18], [41], [23], [-50], [-40], [-14], [-30], [58], [36], [-38], [-23], [41], [-11], [-8], [-38], [8], [4], [-15], [12], [-2], [-14], [-66], [42], [57], [45], [3], [23], [-29], [46], [2], [48], [60], [50], [-7], [40], [-55], [60], [66], [-24], [21], [-70], [32], [-58], [-17], [-57], [48], [30], [42], [68], [-54], [2], [18], [-26], [41], [-60], [31], [8], [-26], [43], [-54], [-31], [-19], [-26], [37], [-38], [-17], [76], [54], [-14], [39], [30], [-45], [16], [44], [-34], [34], [-44], [20], [-12], [3], [-48], [-21], [-42], [-8], [-62], [-46], [-50], [-36], [-3], [17], [-49], [13], [-50], [20], [62], [65], [63], [18], [-6], [9], [-4], [-30], [32], [-51], [44], [-54], [25], [-22], [0], [-10], [50], [-8], [-16], [40], [22], [45], [-5], [36], [19], [42], [-82], [-46], [-58], [-24], [-33], [44], [32], [-84], [-26], [34], [0], [23], [63], [-55], [16], [-8], [81], [19], [44], [-16], [-68], [-56], [-34], [-36], [15], [-76], [48], [-6], [20], [40], [-12], [-70], [55], [-9], [42], [32], [-15], [8], [4], [-64], [-15], [43], [-80], [76], [-9], [-18], [4], [-41], [7], [-12], [54], [48], [-6], [14], [-10], [15], [-66], [71], [-74], [-64], [0], [-90], [20], [-83], [36], [-9], [74], [-38], [-5], [-76], [23], [-4], [-48], [-84], [-56], [95], [40], [-17], [-58], [42], [66], [82], [-82], [-7], [-39], [-40], [-17], [-71], [21], [61], [66], [22], [-21], [-68], [-34], [-65], [-4], [15], [-19], [62], [-58], [-27], [5], [38], [12], [-64], [38], [-58], [-57], [-25], [-56], [3], [-9], [-98], [66], [-14], [-28], [-71], [-81], [-22], [-6], [64], [54], [106], [48], [56], [-6], [-67], [-33], [-74], [30], [22], [86], [-93], [78], [-10], [-14], [81], [-22], [-37], [-10], [20]]
